/**
 * Eine Klasse zum Benchmarken von Java-Functionen
 * @author Patrick Kilian
 */
public class Laufzeit {
	/**
	 * misst die Laufzeit einer Funktion
	 * @param f Funktion deren Laufzeit bestimmt wird
	 * @param x Eingabewert f&uuml;r f
	 * @return Laufzeit in Millisekunden
	 */
	public static int t(UnaryIntFunction f, int x) {
		long start = System.currentTimeMillis();
		f.f(x);
		long ende = System.currentTimeMillis();
		return((int) (ende - start));
	}
	/**
	 * endscheidet ob o(x) eine obere Schranke von f(x) ist, f&uuml;r alle x im
	 * Intervall [a; b]
	 * @param f Funktion unbekannter Laufzeit
	 * @param o m&ouml;gliche obere Schranke
	 * @param a untere Grenze des Intervalles von x-Werten
	 * @param b obere Grenze des Intervalles von x-Werten
	 * @return gilt c*o(x) &gt; f(x) f&uuml;r alle x mit a &le; x &le; b
	 */
	public static boolean laufzeitTest(UnaryIntFunction f, UnaryIntFunction o,
	                                   int a, int b) {
		long start;
		long ende;
		int max;
		for (int i=a; i <= b; i++) {
			max = o.f(i);
			start = System.currentTimeMillis();
			f.f(i);
			ende = System.currentTimeMillis();
			if ((long)max < (ende -start)) {
				return(false);
			}
		}
		return(true);
	}
	/**
	 * berechnet c von c*(x) &gt; f(x) und entscheidet ob c_i &gt; c_(i+1)
	 * @param f Funktion unbekannter Laufzeit
	 * @param o m&ouml;gliche obere Schranke
	 * @param a untere Grenze der x-Werte
	 * @param b obere Grenze der x-Werte
	 * @param schritteweite Abstand zwischen zwei aufeinanderfolgenden x-Werten
	 * @return gilt  c_i &gt; c_(i+1) f&uuml;r alle x mit  a &le; x &le; b
	 */
	public static boolean monotonFallend(UnaryIntFunction f, UnaryIntFunction o,
	                                     int a,	int b, int schrittweite) {

		double c_old;
		double c_new;
		if(b < a) {
			System.out.println("kein Intervall");
			return(false);
		}
		c_old = t(f, a) / o.f(a);
		for (int i = a; i <= b; i+=schrittweite) {
			c_new = t(f, i) / o.f(i);
			if (c_new > c_old) {
				return(false);
			}
			c_old = c_new;
		}
		return(true);
	}
	/**
	 * berechnet c von c*(x) &lt; f(x) und entscheidet ob c_i &lt; c_(i+1)
	 * @param f Funktion unbekannter Laufzeit
	 * @param o m&ouml;gliche obere Schranke
	 * @param a untere Grenze der x-Werte
	 * @param b obere Grenze der x-Werte
	 * @param schritteweite Abstand zwischen zwei aufeinanderfolgenden x-Werten
	 * @return gilt  c_i &lt; c_(i+1) f&uuml;r alle x mit  a &le; x &le; b
	 */
	public static boolean monotonSteigend(UnaryIntFunction f,
	                                      UnaryIntFunction o, int a, int b,
	                                      int schrittweite) {
		double c_old;
		double c_new;
		if(b < a) {
			System.out.println("kein Intervall");
			return(false);
		}
		c_old = t(f, a) / o.f(a);
		for (int i = a; i <= b; i+=schrittweite) {
			c_new = t(f, i) / o.f(i);
			if (c_new < c_old) {
				return(false);
			}
			c_old = c_new;
		}
		return(true);
	}
	/**
	 * Funktion j aus der Aufgabe 4e des letzten &Uuml;bungsblattes
	 */
	private static UnaryIntFunction func_j = new UnaryIntFunction() {
		public int f(int x) {
			if (x < 1) {
				return(1);
			} else {
				return(func_j.f(x/2) + func_j.f(2*(x/3)) + func_j.f(3*(x/4)));
			}
		}
	};
	/**
	 * Schrankenfunction o(x) = log(x)
	 */
	private static UnaryIntFunction func_ologx = new UnaryIntFunction() {
		public int f(int x) {
			return((int) Math.max(Math.log(x), 1));
		}
	};
	/**
	 * Schrankenfunction o(x) = x
	 */
	private static UnaryIntFunction func_ox = new UnaryIntFunction() {
		public int f(int x) {
			return(x);
		}
	};
	/**
	 * Schrankenfunction o(x) = x*x
	 */
	private static UnaryIntFunction func_ox2 = new UnaryIntFunction() {
		public int f(int x) {
			return(x*x);
		}
	};
	/**
	 * main-Function zum Testen
	 * @param args werden ignoriert
	 */
	public static void main(String[] args) {
		System.out.println("j(   1) "+t(func_j, 1));
		System.out.println("j( 250) "+t(func_j, 250));
		System.out.println("j( 500) "+t(func_j, 500));
		System.out.println("j( 750) "+t(func_j, 750));
		System.out.println("j(1000) "+t(func_j, 1000));
		System.out.println("j(1250) "+t(func_j, 1250));
		System.out.println("j(1500) "+t(func_j, 1500));
		System.out.println("j(1750) "+t(func_j, 1750));
		// Timeout
		System.out.println("monotonFallend(j, o(log(x)), 2, 2000, 10)");
		System.out.println(monotonFallend(func_j, func_ologx, 2, 2000, 10));

		System.out.println("monotonFallend(j, o(x), 1, 2000, 10)");
		System.out.println(monotonFallend(func_j, func_ox, 1, 2000, 10));

		System.out.println("monotonFallend(j, o(x^2), 1, 2000, 10)");
		System.out.println(monotonFallend(func_j, func_ox2, 1, 2000, 10));
		
	}
}
